@@ -22,7 +22,7 @@
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Thomas Perl <m@thp.io>'
-__version__ = '0.6.8'
+__version__ = '0.6.11'
 __website__ = 'http://gpodder.org/podcastparser/'
 __license__ = 'ISC License'
 
@@ -32,38 +32,10 @@ import re
 import os
 import time
 
-try:
-    # Python 2
-    from htmlentitydefs import entitydefs
-    entitydefs = dict((key, value.decode('latin-1')) for key, value in entitydefs.iteritems())
-    chr = unichr
-except ImportError:
-    # Python 3
-    from html.entities import entitydefs
+from html.entities import entitydefs
 
-try:
-    # Python 2
-    import urlparse
-except ImportError:
-    # Python 3
-    from urllib import parse as urlparse
-
-try:
-    # Python 2
-    from rfc822 import parsedate_tz
-    import calendar
-    # This is taken from Python 3's email._parseaddr, since it handles
-    # pre-epoch dates better than what Python 2 does (time.mktime())
-    def mktime_tz(data):
-        if data[9] is None:
-            # No zone info, so localtime is better assumption than GMT
-            return time.mktime(data[:8] + (-1,))
-        else:
-            t = calendar.timegm(data)
-            return t - data[9]
-except ImportError:
-    # Python 3
-    from email.utils import mktime_tz, parsedate_tz
+from urllib import parse as urlparse
+from email.utils import mktime_tz, parsedate_tz
 
 import logging
 logger = logging.getLogger(__name__)
@@ -104,13 +76,25 @@ class PodcastAttr(Target):
     WANT_TEXT = True
 
     def end(self, handler, text):
+        if not self.overwrite and handler.get_podcast_attr(self.key):
+            return
         handler.set_podcast_attr(self.key, self.filter_func(text))
 
+class PodcastAttrList(Target):
+    WANT_TEXT = True
+
+    def end(self, handler, text):
+        if not self.overwrite and handler.get_podcast_attr(self.key):
+            return
+        handler.set_podcast_attr(self.key, self.filter_func(text).split(', '))
+
 
 class PodcastAttrType(Target):
     WANT_TEXT = True
 
     def end(self, handler, text):
+        if not self.overwrite and handler.get_podcast_attr(self.key):
+            return
         value = self.filter_func(text)
         if value in ('episodic', 'serial'):
             handler.set_podcast_attr(self.key, value)
@@ -123,13 +107,21 @@ class PodcastAttrRelativeLink(PodcastAttr):
 
 
 class PodcastAttrFromHref(Target):
+    ATTRIBUTE = 'href'
+    
     def start(self, handler, attrs):
-        value = attrs.get('href')
+        if not self.overwrite and handler.get_podcast_attr(self.key):
+            return
+        value = attrs.get(self.ATTRIBUTE)
         if value:
             value = urlparse.urljoin(handler.base, value)
             handler.set_podcast_attr(self.key, self.filter_func(value))
 
 
+class PodcastAttrFromUrl(PodcastAttrFromHref):
+    ATTRIBUTE = 'url'
+    
+
 class EpisodeItem(Target):
     def start(self, handler, attrs):
         handler.add_episode()
@@ -144,7 +136,9 @@ class EpisodeAttr(Target):
     def end(self, handler, text):
         if not self.overwrite and handler.get_episode_attr(self.key):
             return
-        handler.set_episode_attr(self.key, self.filter_func(text))
+        text = self.filter_func(text)
+        if text != '' or handler.get_episode_attr(self.key) is None:
+            handler.set_episode_attr(self.key, text)
 
 
 class EpisodeAttrRelativeLink(EpisodeAttr):
@@ -185,6 +179,15 @@ class EpisodeAttrFromUrl(EpisodeAttrFromHref):
     ATTRIBUTE = 'url'
 
 
+class EpisodeAttrSeason(EpisodeAttr):
+    def end(self, handler, text):
+        try:
+            episode_season = int(text)
+        except ValueError:
+            episode_season = 0
+        handler.set_episode_attr(self.key, episode_season)
+
+
 class EpisodeAttrNumber(EpisodeAttr):
     def end(self, handler, text):
         value = self.filter_func(text)
@@ -320,6 +323,31 @@ class PodloveChapter(Target):
                 chapter[optional] = value
 
         handler.get_episode_attr('chapters').append(chapter)
+        
+        
+class EpisodePersonAttr(Target):
+    WANT_TEXT = True
+         
+    def start(self, handler, attrs):
+        if not handler.get_episode_attr("persons"):
+            handler.add_episode_persons()
+        person = {
+            "name": None,
+            "role": "host",
+            "group": "cast",
+            "href": None,
+            "img": None,
+        }
+        for optional in ("group", "role", "href", "img"):
+            value = attrs.get(optional)
+            if value:
+                if optional in ("role", "group"):
+                    value = value.lower()
+                person[optional] = value
+        handler.append_episode_person(person)
+    
+    def end(self, handler, text):
+        handler.get_episode_attr("persons")[-1]["name"] = text
 
 
 class ItunesOwnerAttr(Target):
@@ -331,6 +359,22 @@ class ItunesOwnerAttr(Target):
         handler.append_itunes_owner(self.key, self.filter_func(text))
 
 
+class ItunesCategoryAttr(Target):
+    def start(self, handler, attrs):
+        # Let's use an empty string as a fallback for first-level categories
+        # in case there is a valid sub-category.
+        value = attrs.get('text', '')
+        handler.append_itunes_category(self.key, self.filter_func(value))
+
+
+class ItunesSubCategoryAttr(Target):
+    def start(self, handler, attrs):
+        value = attrs.get('text')
+        if not value:
+            return
+        handler.append_itunes_subcategory(self.key, self.filter_func(value))
+
+
 class ItunesOwnerItem(Target):
     def start(self, handler, attrs):
         handler.add_itunes_owner()
@@ -385,6 +429,11 @@ class Namespace():
 
         # Purl RSS Content module
         'http://purl.org/rss/1.0/modules/content/': 'content',
+        
+        # Podcast Index podcast namespace
+        # https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md
+        'https://github.com/Podcastindex-org/podcast-namespace/blob/main/docs/1.0.md': 'podcast',
+        'https://github.com/podcastindex-org/podcast-namespace/blob/main/docs/1.0.md': 'podcast',
     }
 
     def __init__(self, attrs, parent=None):
@@ -658,8 +707,8 @@ def parse_pubdate(text):
         try:
             pubtimeseconds = int(mktime_tz(parsed))
             return pubtimeseconds
-        except(OverflowError,ValueError):
-            logger.warning('bad pubdate %s is before epoch or after end of time (2038)',parsed)
+        except(OverflowError, ValueError):
+            logger.warning('bad pubdate %s is before epoch or after end of time (2038)', parsed)
             return 0
 
     try:
@@ -693,16 +742,25 @@ MAPPING = {
     'rss/channel/title': PodcastAttr('title', squash_whitespace),
     'rss/channel/link': PodcastAttrRelativeLink('link'),
     'rss/channel/description': PodcastAttr('description', squash_whitespace_not_nl),
+    'rss/channel/itunes:summary': PodcastAttr('description', squash_whitespace_not_nl, overwrite=False),
+    'rss/channel/podcast:funding': PodcastAttrFromUrl('funding_url'),
+    'rss/channel/podcast:locked': PodcastAttrExplicit('import_prohibited'),
     'rss/channel/image/url': PodcastAttrRelativeLink('cover_url'),
     'rss/channel/itunes:image': PodcastAttrFromHref('cover_url'),
     'rss/channel/itunes:type': PodcastAttrType('type', squash_whitespace),
     'rss/channel/atom:link': PodcastAtomLink(),
     'rss/channel/generator': PodcastAttr('generator', squash_whitespace),
     'rss/channel/language': PodcastAttr('language', squash_whitespace),
+
+    'rss/channel/itunes:category': ItunesCategoryAttr('itunes_categories'),
+    'rss/channel/itunes:category/itunes:category': ItunesSubCategoryAttr('itunes_categories'),
+    'rss/channel/itunes:category/itunes:category/itunes:category': ItunesSubCategoryAttr('itunes_categories'),
+
     'rss/channel/itunes:author': PodcastAttr('itunes_author', squash_whitespace),
     'rss/channel/itunes:owner': ItunesOwnerItem('itunes_owner', squash_whitespace),
     'rss/channel/itunes:explicit': PodcastAttrExplicit('explicit', squash_whitespace),
     'rss/channel/itunes:new-feed-url': PodcastAttr('new_url', squash_whitespace),
+    'rss/channel/itunes:keywords': PodcastAttrList('itunes_keywords', squash_whitespace),
     'rss/redirect/newLocation': PodcastAttr('new_url', squash_whitespace),
 
     'rss/channel/itunes:owner/itunes:email': ItunesOwnerAttr('email', squash_whitespace),
@@ -722,6 +780,7 @@ MAPPING = {
     'rss/channel/item/atom:link': AtomLink(),
     'rss/channel/item/itunes:explicit': EpisodeAttrExplicit('explicit', squash_whitespace),
     'rss/channel/item/itunes:author': EpisodeAttr('itunes_author', squash_whitespace),
+    'rss/channel/item/itunes:season': EpisodeAttrSeason('season', squash_whitespace),
     'rss/channel/item/itunes:episode': EpisodeAttrNumber('number', squash_whitespace),
     'rss/channel/item/itunes:episodeType': EpisodeAttrType('type', squash_whitespace),
 
@@ -734,6 +793,9 @@ MAPPING = {
     'rss/channel/item/enclosure': Enclosure('length'),
     'rss/channel/item/psc:chapters': PodloveChapters(),
     'rss/channel/item/psc:chapters/psc:chapter': PodloveChapter(),
+    'rss/channel/item/podcast:transcript': EpisodeAttrFromUrl("transcript_url"),
+    'rss/channel/item/podcast:chapters': EpisodeAttrFromUrl("chapters_json_url"),
+    'rss/channel/item/podcast:person': EpisodePersonAttr(),
 
     # Basic support for Atom feeds
     'atom:feed': PodcastItem(),
@@ -792,11 +854,20 @@ class PodcastHandler(sax.handler.ContentHandler):
     def set_podcast_attr(self, key, value):
         self.data[key] = value
 
+    def get_podcast_attr(self, key, default=None):
+        return self.data.get(key, default)
+
     def set_episode_attr(self, key, value):
         self.episodes[-1][key] = value
 
     def get_episode_attr(self, key, default=None):
         return self.episodes[-1].get(key, default)
+        
+    def add_episode_persons(self):
+        self.episodes[-1]['persons'] = []
+        
+    def append_episode_person(self, value):
+        self.episodes[-1]['persons'].append(value)
 
     def add_episode(self):
         self.episodes.append({
@@ -868,6 +939,16 @@ class PodcastHandler(sax.handler.ContentHandler):
     def append_itunes_owner(self, key, value):
         self.data['itunes_owner'][key] = value
 
+    def add_itunes_categories(self):
+        self.data['itunes_categories'] = []
+
+    def append_itunes_category(self, key, value):
+        self.data.setdefault('itunes_categories', []).append([value])
+
+    def append_itunes_subcategory(self, key, value):
+        entry = self.data['itunes_categories'][-1]
+        entry.append(value)
+
     def startElement(self, name, attrs):
         self.namespace = Namespace(attrs, self.namespace)
         name = self.namespace.map(name)
@@ -988,7 +1069,10 @@ def normalize_feed_url(url):
     # urlunsplit might return "a slighty different, but equivalent URL"
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
+
 HTML_TEST = re.compile(r'<[a-z][a-z0-9]*(?:\s.*?>|\/?>)', re.IGNORECASE | re.DOTALL)
+
+
 def is_html(text):
     """Heuristically tell if text is HTML
 
@@ -1000,6 +1084,7 @@ def is_html(text):
     """
     return bool(HTML_TEST.search(text))
 
+
 def remove_html_tags(html):
     """
     Remove HTML tags from a string and replace numeric and
